# # -*- coding: utf-8 -*-

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.models import ContentType
from django_extensions.db import fields as exfields


class BaseLinkProperty(models.Model):
    label = models.CharField(_(u'Label'), max_length=100)
    uri = models.URLField(blank=True, verbose_name=_(u'property URI'))  # verify_exists=False,

    class Meta:
        abstract = True
        verbose_name = _(u'linking property')
        verbose_name_plural = _(u'linking properties')
        app_label = 'coop_local'

    def __unicode__(self):
        return self.label


class BaseLink(models.Model):
    """
    generic link model, recording some properties about an object URI
    SeeAlso or SameAs
    """
    created = exfields.CreationDateTimeField(_(u'created'), null=True)
    modified = exfields.ModificationDateTimeField(_(u'modified'), null=True)
    content_type = models.ForeignKey(ContentType, blank=True, null=True)
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('content_type', 'object_id')

    predicate = models.ForeignKey('coop_local.LinkProperty')
    object_uri = models.URLField(blank=True, verbose_name=_(u'object URI'))  # verify_exists=False, 

    def __unicode__(self):
        return u"%s %s %s" % (unicode(self.content_object), unicode(self.predicate), self.object_uri)

    class Meta:
        abstract = True
        verbose_name = _(u'semantic link')
        verbose_name_plural = _(u'semantic links')
        app_label = 'coop_local'





"""
Convenience forms for adding and updating ``Event`` and ``Occurrence``s.
"""
from datetime import datetime, date, time, timedelta

from django import forms
from django.utils.translation import ugettext_lazy as _
from django.forms.extras.widgets import SelectDateWidget

from dateutil import rrule
from coop.agenda.conf import settings as agenda_settings
from coop.agenda import utils
from coop_local.models import Occurrence
from django.db.models.loading import get_model

WEEKDAY_SHORT = (
    (7, _(u'Sun')),
    (1, _(u'Mon')),
    (2, _(u'Tue')),
    (3, _(u'Wed')),
    (4, _(u'Thu')),
    (5, _(u'Fri')),
    (6, _(u'Sat'))
)

WEEKDAY_LONG = (
    (7, _(u'Sunday')),
    (1, _(u'Monday')),
    (2, _(u'Tuesday')),
    (3, _(u'Wednesday')),
    (4, _(u'Thursday')),
    (5, _(u'Friday')),
    (6, _(u'Saturday'))
)

MONTH_LONG = (
    (1,  _(u'January')),
    (2,  _(u'February')),
    (3,  _(u'March')),
    (4,  _(u'April')),
    (5,  _(u'May')),
    (6,  _(u'June')),
    (7,  _(u'July')),
    (8,  _(u'August')),
    (9,  _(u'September')),
    (10, _(u'October')),
    (11, _(u'November')),
    (12, _(u'December')),
)

MONTH_SHORT = (
    (1,  _(u'Jan')),
    (2,  _(u'Feb')),
    (3,  _(u'Mar')),
    (4,  _(u'Apr')),
    (5,  _(u'May')),
    (6,  _(u'Jun')),
    (7,  _(u'Jul')),
    (8,  _(u'Aug')),
    (9,  _(u'Sep')),
    (10, _(u'Oct')),
    (11, _(u'Nov')),
    (12, _(u'Dec')),
)


ORDINAL = (
    (1,  _(u'first')),
    (2,  _(u'second')),
    (3,  _(u'third')),
    (4,  _(u'fourth')),
    (-1, _(u'last'))
)

FREQUENCY_CHOICES = (
    (rrule.DAILY,   _(u'Day(s)')),
    (rrule.WEEKLY,  _(u'Week(s)')),
    (rrule.MONTHLY, _(u'Month(s)')),
    (rrule.YEARLY,  _(u'Year(s)')),
)

REPEAT_CHOICES = (
    ('count', _(u'By count')),
    ('until', _(u'Until date')),
)

ISO_WEEKDAYS_MAP = (
    None,
    rrule.MO,
    rrule.TU,
    rrule.WE,
    rrule.TH,
    rrule.FR,
    rrule.SA,
    rrule.SU
)

MINUTES_INTERVAL = agenda_settings.TIMESLOT_INTERVAL.seconds // 60
SECONDS_INTERVAL = utils.time_delta_total_seconds(agenda_settings.DEFAULT_OCCURRENCE_DURATION)


def timeslot_options(interval=agenda_settings.TIMESLOT_INTERVAL,
    start_time=agenda_settings.TIMESLOT_START_TIME,
    end_delta=agenda_settings.TIMESLOT_END_TIME_DURATION,
    fmt=agenda_settings.TIMESLOT_TIME_FORMAT):
    """
    Create a list of time slot options for use in agenda forms.

    The list is comprised of 2-tuples containing a 24-hour time value and a
    12-hour temporal representation of that offset.
    """
    dt = datetime.combine(date.today(), time(0))
    dtstart = datetime.combine(dt.date(), start_time)
    dtend = dtstart + end_delta
    options = []

    while dtstart <= dtend:
        options.append((str(dtstart.time()), dtstart.strftime(fmt)))
        dtstart += interval

    return options


def timeslot_offset_options(interval=agenda_settings.TIMESLOT_INTERVAL,
    start_time=agenda_settings.TIMESLOT_START_TIME,
    end_delta=agenda_settings.TIMESLOT_END_TIME_DURATION,
    fmt=agenda_settings.TIMESLOT_TIME_FORMAT):
    """
    Create a list of time slot options for use in agenda forms.

    The list is comprised of 2-tuples containing the number of seconds since the
    start of the day and a 12-hour temporal representation of that offset.
    """
    dt = datetime.combine(date.today(), time(0))
    dtstart = datetime.combine(dt.date(), start_time)
    dtend = dtstart + end_delta
    options = []

    delta = utils.time_delta_total_seconds(dtstart - dt)
    seconds = utils.time_delta_total_seconds(interval)
    while dtstart <= dtend:
        options.append((delta, dtstart.strftime(fmt)))
        dtstart += interval
        delta += seconds

    return options

default_timeslot_options = timeslot_options()
default_timeslot_offset_options = timeslot_offset_options()


class MultipleIntegerField(forms.MultipleChoiceField):
    """
    A form field for handling multiple integers.
    """

    def __init__(self, choices, size=None, label=None, widget=None):
        if widget is None:
            widget = forms.SelectMultiple(attrs={'size': size or len(choices)})
        super(MultipleIntegerField, self).__init__(
            required=False,
            choices=choices,
            label=label,
            widget=widget,
        )

    def clean(self, value):
        return [int(i) for i in super(MultipleIntegerField, self).clean(value)]


class SplitDateTimeWidget(forms.MultiWidget):
    """
    A Widget that splits datetime input into a SelectDateWidget for dates and
    Select widget for times.
    """

    def __init__(self, attrs=None):
        widgets = (
            SelectDateWidget(attrs=attrs),
            forms.Select(choices=default_timeslot_options, attrs=attrs)
        )
        super(SplitDateTimeWidget, self).__init__(widgets, attrs)

    def decompress(self, value):
        if value:
            return [value.date(), value.time().replace(microsecond=0)]

        return [None, None]


class MultipleOccurrenceForm(forms.Form):
    day = forms.DateField(
        label=_(u'Date'),
        initial=date.today,
        widget=SelectDateWidget(),
        localize=True,
    )

    start_time_delta = forms.IntegerField(
        label=_(u'Start time'),
        widget=forms.Select(choices=default_timeslot_offset_options)
    )

    end_time_delta = forms.IntegerField(
        label=_(u'End time'),
        widget=forms.Select(choices=default_timeslot_offset_options)
    )

    # recurrence options
    repeats = forms.ChoiceField(
        choices=REPEAT_CHOICES,
        initial='count',
        label=_(u'Occurrences'),
        widget=forms.RadioSelect(),
        localize=True,
    )

    count = forms.IntegerField(
        label=_(u'Total Occurrences'),
        initial=1,
        required=False,
        widget=forms.TextInput(attrs=dict(size=2, max_length=2))
    )

    until = forms.DateField(
        required=False,
        initial=date.today,
        widget=SelectDateWidget(),
        localize=True,
    )

    freq = forms.IntegerField(
        label=_(u'Frequency'),
        initial=rrule.WEEKLY,
        widget=forms.RadioSelect(choices=FREQUENCY_CHOICES),
    )

    interval = forms.IntegerField(
        required=False,
        initial='1',
        widget=forms.TextInput(attrs=dict(size=3, max_length=3))
    )

    # weekly options
    week_days = MultipleIntegerField(
        WEEKDAY_SHORT,
        label=_(u'Weekly options'),
        widget=forms.CheckboxSelectMultiple
    )

    # monthly  options
    month_option = forms.ChoiceField(
        choices=(('on', _(u'On the')), ('each', _(u'Each:'))),
        initial='each',
        widget=forms.RadioSelect(),
        label=_(u'Monthly options'),
        localize=True,
    )

    month_ordinal = forms.IntegerField(widget=forms.Select(choices=ORDINAL))
    month_ordinal_day = forms.IntegerField(widget=forms.Select(choices=WEEKDAY_LONG))
    each_month_day = MultipleIntegerField(
        [(i, i) for i in range(1, 32)],
        widget=forms.CheckboxSelectMultiple
    )

    # yearly options
    year_months = MultipleIntegerField(
        MONTH_SHORT,
        label=_(u'Yearly options'),
        widget=forms.CheckboxSelectMultiple
    )

    is_year_month_ordinal = forms.BooleanField(required=False)
    year_month_ordinal = forms.IntegerField(widget=forms.Select(choices=ORDINAL))
    year_month_ordinal_day = forms.IntegerField(widget=forms.Select(choices=WEEKDAY_LONG))

    def __init__(self, *args, **kws):
        super(MultipleOccurrenceForm, self).__init__(*args, **kws)
        dtstart = self.initial.get('dtstart', None)
        if dtstart:
            dtstart = dtstart.replace(
                minute=((dtstart.minute // MINUTES_INTERVAL) * MINUTES_INTERVAL),
                second=0,
                microsecond=0
            )

            weekday = dtstart.isoweekday()
            ordinal = dtstart.day // 7
            ordinal = u'%d' % (-1 if ordinal > 3 else ordinal + 1,)

            self.initial.setdefault('week_days', u'%d' % weekday)
            self.initial.setdefault('month_ordinal', ordinal)
            self.initial.setdefault('month_ordinal_day', u'%d' % weekday)
            self.initial.setdefault('each_month_day', [u'%d' % dtstart.day])
            self.initial.setdefault('year_months', [u'%d' % dtstart.month])
            self.initial.setdefault('year_month_ordinal', ordinal)
            self.initial.setdefault('year_month_ordinal_day', u'%d' % weekday)

            offset = (dtstart - datetime.combine(dtstart.date(), time(0))).seconds
            self.initial.setdefault('start_time_delta', u'%d' % offset)
            self.initial.setdefault('end_time_delta', u'%d' % (offset + SECONDS_INTERVAL,))

    def clean(self):
        day = datetime.combine(self.cleaned_data['day'], time(0))
        self.cleaned_data['start_time'] = day + timedelta(
            seconds=self.cleaned_data['start_time_delta']
        )

        self.cleaned_data['end_time'] = day + timedelta(
            seconds=self.cleaned_data['end_time_delta']
        )

        return self.cleaned_data

    def save(self, event):
        if self.cleaned_data['repeats'] == 'no':
            params = {}
        else:
            params = self._build_rrule_params()

        event.add_occurrences(
            self.cleaned_data['start_time'],
            self.cleaned_data['end_time'],
            **params
        )

        return event

    def _build_rrule_params(self):
        iso = ISO_WEEKDAYS_MAP
        data = self.cleaned_data
        params = dict(
            freq=data['freq'],
            interval=data['interval'] or 1
        )

        if self.cleaned_data['repeats'] == 'count':
            params['count'] = data['count']
        elif self.cleaned_data['repeats'] == 'until':
            params['until'] = data['until']

        if params['freq'] == rrule.WEEKLY:
            params['byweekday'] = [iso[n] for n in data['week_days']]

        elif params['freq'] == rrule.MONTHLY:
            if 'on' == data['month_option']:
                ordinal = data['month_ordinal']
                day = iso[data['month_ordinal_day']]
                params['byweekday'] = day(ordinal)
            else:
                params['bymonthday'] = data['each_month_day']

        elif params['freq'] == rrule.YEARLY:
            params['bymonth'] = data['year_months']
            if data['is_year_month_ordinal']:
                ordinal = data['year_month_ordinal']
                day = iso[data['year_month_ordinal_day']]
                params['byweekday'] = day(ordinal)

        elif params['freq'] != rrule.DAILY:
            raise NotImplementedError(_(u'Unknown interval rule %s') % params['freq'])

        return params


class EventForm(forms.ModelForm):
    """
    A simple form for adding and updating Event attributes
    """

    class Meta:
        model = get_model('coop_local', 'Event')

    def __init__(self, *args, **kws):
        super(EventForm, self).__init__(*args, **kws)
        self.fields['description'].required = False


class SingleOccurrenceForm(forms.ModelForm):
    """
    A simple form for adding and updating single Occurrence attributes
    """

    start_time = forms.DateTimeField(widget=SplitDateTimeWidget, localize=True,)
    end_time = forms.DateTimeField(widget=SplitDateTimeWidget, localize=True,)

    class Meta:
        model = Occurrence



# -*- coding:utf-8 -*-
from django.db import models
from extended_choices import Choices


# Use Choices() !
# ETAT = ((0, 'Inconnu'),
#         (1, 'Active'),
#         (2, 'En sommeil'),
#         (3, 'En projet'),
# )

# Here you can either :
# - Customize coop models by deriving from the abstract class (BaseOrganization, Baseperson...)
# - Or add your own models, providing you add in their Meta class app_label="coop_local"


# -*- coding:utf-8 -*-
from django import forms
from django.contrib import admin
from django.core.exceptions import ImproperlyConfigured

try:
    from coop.base_admin import *
except ImportError, exp:
    raise ImproperlyConfigured("Unable to find coop/base_admin.py file")


# subclass existing ModelAdmins and add your own model's ModelAdmins here

"""
# ---- overriding main models when needed : example -----------------------

# first unregister previous ModelAdmin
admin.site.unregister(Person)

# define anothe admin
class MyPersonAdmin(PersonAdmin):
    Add custom fields you defined in coop_local.models
    fieldsets = (
        ('Identification', {
            'fields': (('first_name', 'last_name'),
                        ('location', 'location_display'),  # Using coop-geo
                        'email',
                        'category'
                        ),
            }),
        ('Notes', {
            'fields': ('structure', 'notes',)
        })
    )
    Using coop-geo
    related_search_fields = {'location': ('label', 'adr1', 'adr2', 'zipcode', 'city'), }
admin.site.register(Person, MyPersonAdmin)

# ----- admin for classifications : ultra-simple -----------------------------

admin.site.register(Statut, CoopTagTreeAdmin)



"""



# -*- coding:utf-8 -*-

from django.conf.urls.defaults import *
#from django.views.generic import ListView
#from coop.org.views import ISDetailView
#from coop_local.models import Organization


urlpatterns = patterns('',

    url(r'^$', 'coop.org.views.org_list', name="org_list"),
    url(r'^carto/$', 'coop.org.views.leaflet', name="leaflet"),

    url(r'^categorie/(?P<slug>[\w-]+)/$', 'coop.org.views.org_category_detail', name="org_category_detail"),

    url(r'^global_map$', 'coop.org.views.global_map', name="org_global_map"),
    url(r'^(?P<slug>[\w-]+)/$', 'coop.org.views.org_detail_view', name="org_detail"),
    url(r'^(?P<slug>[\w-]+)/edit/$', 'coop.org.views.org_edit', name="org_edit"),

    #url(r'^role/(?P<slug>[\w-]+)/$', 'coop.org.views.role_detail', name="role_detail"),


)



# -*- coding: utf-8 -*-
from fabric.api import env, local, run, sudo, cd, hide, show, settings, prefix, prompt
from fabric.contrib.console import confirm
from fabric.colors import red, green, yellow
from fabric.decorators import task
from fabric.contrib.files import append, contains, exists, sed, upload_template
from fabtools import icanhaz
import fabtools

#Paramétres Vagrant
env.vm_path = "/Users/dom/VM/devcoop"

#Paramétres serveur
env.locale = 'fr_FR.UTF-8'

#Paramétres par défaut
domaine = "localhost"
projet = "coop_test"
alias = "coop_test"

pgpass = '123456'

# Paramétres Déploiement
env.websrv = 1


def pretty_apt(pkglist):
    for pkg in (pkglist):
        icanhaz.deb.package(pkg)
        print(green(u'Paquet Debian "' + unicode(pkg) + u'" : installé.'))


def pretty_pip(pkglist):
    for pkg in (pkglist):
        icanhaz.python.install(pkg)
        print(green(u'Module Python "' + unicode(pkg) + u'" : installé.'))

@task
def local_vm():
    '''First command to use for a Vagrant VM'''
    # change from the default user to 'vagrant'
    env.user = 'vagrant'
    # connect to the port-forwarded ssh
    env.hosts = ['127.0.0.1:2222']
    # use vagrant ssh key
    #with cd(env.vm_path):
    result = local('vagrant ssh_config | grep IdentityFile', capture=True)
    env.key_filename = result.split()[1]

    # si le partage de dossier ne marche pas bien encore ensuite :
    #mise à jour guest additions
    #icanhaz.deb.packages(['dkms','linux-headers-2.6.38-8-generic','linux-headers-generic'])
    # sudo('/etc/init.d/vboxadd setup') # puis "vagrant reload"


@task
def remote():
    '''First command to use for a SSH remote host'''
    env.hosts = [prompt('Alias SSH:', default=alias)]

    def _annotate_hosts_with_ssh_config_info():
        from os.path import expanduser
        from paramiko.config import SSHConfig

        def hostinfo(host, config):
            hive = config.lookup(host)
            if 'hostname' in hive:
                host = hive['hostname']
            if 'user' in hive:
                host = '%s@%s' % (hive['user'], host)
            if 'port' in hive:
                host = '%s:%s' % (host, hive['port'])
            return host
        try:
            config_file = file(expanduser('~/.ssh/config'))
        except IOError:
            pass
        else:
            config = SSHConfig()
            config.parse(config_file)
            keys = [config.lookup(host).get('identityfile', None)
                for host in env.hosts]
            env.key_filename = [expanduser(key) for key in keys if key is not None]
            env.hosts = [hostinfo(host, config) for host in env.hosts]
    _annotate_hosts_with_ssh_config_info()
    #sudo apt-get install nano
    #update-alternatives  --config editor
    #visudo
    #Dans le fichier, rendez vous à la ligne %admin ALL=(ALL) ALL.
    #Remplacez %admin ALL=(ALL) ALL par %admin ALL=(ALL) NOPASSWD: ALL


def project():
    if 'projet' not in env.keys():
        prompt('Nom du projet :', default=projet, key='projet')


def domain():
    if 'domaine' not in env.keys():
        prompt('DNS:', default=domaine, key='domaine')

#prompt('Config : (1)Apache seul ou (2)Apache+Nginx :',key='websrv',validate=int,default=env.websrv)


def gandi():
    '''Ajoute le dépôt APT Gandi'''
    with hide('running', 'stdout', 'stderr'):
        run('wget http://mirrors.gandi.net/gandi/pubkey')
        sudo('apt-key add pubkey')
        print(green('Gandi public key added.'))


def pip_bootstrap():
    with cd("/tmp"):
        run("curl --silent -O https://raw.github.com/pypa/pip/master/contrib/get-pip.py")
        sudo("python get-pip.py")


@task
def server_setup():
    '''Installation serveur pour Ubuntu >= 10.10'''
    with settings(show('user'), hide('warnings', 'running', 'stdout', 'stderr')):
        project()
        domain()
        #if gandi
        locale()
        sudo('apt-get -y install aptitude')  # demande un input
        print(yellow('Mise à jour de l’index APT...'))
        fabtools.deb.update_index()  # apt-get quiet update
        print(yellow('Mise à jour des paquets debian installés...'))
        fabtools.deb.upgrade()
        # paquets communs à tous les serveurs Django+geodjango
        print(yellow('Installation des paquets de base...'))
        pretty_apt(['git-core', 'mercurial', 'gcc', 'curl', 'build-essential',
                    'libfreetype6', 'libfreetype6-dev', 'liblcms1-dev', 'libpng12-dev',
                    'libjpeg8-dev', 'python-imaging',
                    'python-setuptools', 'nano', 'python-dev', 'swig',
                    'memcached', 'python-memcache'])

        # pip special case
        if not fabtools.python.is_pip_installed():
            fabtools.python.install_pip()
            print(green('pip : installé.'))

        virtualenv_setup()
        #config apache
        if(env.websrv == 1):
            apache_setup()
        elif(env.websrv == 2):
            apache_nginx()

        postgresql()


def postgresql():
    '''PostgreSQL 9.1 + PostGIS 1.5'''
    with settings(show('user'), hide('warnings', 'running', 'stdout', 'stderr')):
        project()
        if 'pgpass' not in env.keys():
            prompt('Passe PostgreSQL :', default=pgpass, key='pgpass')
        print(yellow('Configuration PostgreSQL+PostGIS...'))
        pretty_apt(['postgresql', 'binutils', 'gdal-bin', 'libproj-dev', 'postgresql-9.1-postgis',
                    'postgresql-server-dev-9.1', 'python-psycopg2', 'libgeoip1'])
        # print(yellow('Upgrading all packages...'))
        # fabtools.deb.upgrade()
        # création d'un utilisateur postgresql avec le meme nom d'utilisateur
        if not fabtools.postgres.user_exists(env.user):
            fabtools.postgres.create_user(env.user, env.pgpass)
            sudo('''psql -c "ALTER ROLE %(user)s CREATEDB;"''' % env, user='postgres')
            sudo('''psql -c "ALTER USER %(user)s with SUPERUSER;"''' % env, user='postgres')
            print(green('Création d’un superuser "%(user)s" PostgreSQL.' % env))
        if not exists('.pgpass'):
            run('echo "*:*:*:%(user)s:%(pgpass)s" >> .pgpass' % env)
            sudo('chmod 0600 .pgpass')
            print(green('Création du fichier .pgpass.'))
        run('curl https://docs.djangoproject.com/en/dev/_downloads/create_template_postgis-debian.sh -o postgis.sh')
        run('chmod +x postgis.sh')
        run('./postgis.sh')
        #postgresql_net_access()
        icanhaz.postgres.server()  # start server


def project_setup():
    # recolte des variables
    project()
    domain()
    locale()

    # creation du projet django (startproject)
    django_project()
    apache_vhost()
    django_wsgi()

    #créer la db avec le nom du projet (idempotent)
    create_pg_db()
    dependencies()
    sudo('apachectl restart')


@task
def coop_setup():
    """Creation d'un nouveau projet django-coop"""
    project()
    domain()
    locale()
    coop_project()
    dependencies()
    apache_vhost()
    create_pg_db()
    sudo('apachectl restart')


def coop_project():
    '''Créer un projet django dans son virtualenv'''
    project()
    domain()
    with settings(show('user'), hide('warnings', 'running', 'stdout', 'stderr')):
        if not exists('/home/%(user)s/.virtualenvs/%(projet)s' % env):
            # if confirm('Pas de virtualenv "%(projet)s", faut-il le créer ?' % env, default=False):
            run('mkvirtualenv %(projet)s' % env)
            run('source .bash_profile')
        with prefix('workon %(projet)s' % env):
            run('pip install django-coop')
            print(green('Django et django-coop installé.'))
        if not exists('projects/%s/' % (env.projet)):
            print(yellow('le projet %(projet)s n’existe pas encore' % env))
            if confirm('Créer un projet django nommé "%(projet)s" ?' % env,
                        default=False):
                with cd('projects/'):
                    with prefix('workon %(projet)s' % env):
                        run('coop-admin.py startproject %(projet)s --domain %(domaine)s' % env)
                        print(green('Projet Django-coop "%(projet)s" : Installé.' % env))
                        # coop-admin scripts creates the WSGI script so we won't call django_wsgi()
        else:
            print(yellow('Projet Django-coop nommé "%(projet)s" : déjà installé.' % env))


def apache_vhost():
    '''Configuration Vhost apache'''
    project()
    domain()
    #with prefix('workon %(projet)s' % env):
    if(env.websrv == 1):
        vhost_context = {
            'user': env.user,
            'domain': env.domaine,
            'projet': env.projet
        }
        import coop
        coop_path = coop.__path__[0]
        upload_template('%s/fab_templates/vhost.txt' % coop_path,
                        '/etc/apache2/sites-available/%(domaine)s' % env,
                        context=vhost_context, use_sudo=True)
        with cd('/etc/apache2/'):
            with settings(hide('warnings', 'running', 'stdout', 'stderr')):
                sudo('rm sites-enabled/%(domaine)s' % env)
            sudo('ln -s `pwd`/sites-available/%(domaine)s sites-enabled/%(domaine)s' % env)
            print(green('VirtualHost Apache pour %(domaine)s : OK.' % env))
    elif(env.websrv == 2):
        print(red('Script de déploiement pas encore écrit !!!'))  # TODO
    sudo('apachectl restart')


def django_wsgi():
    '''paramétrage WSGI/Apache'''
    project()
    if not exists('/home/%(user)s/%(projet)s/coop_local/wsgi.py' % env):
        with prefix('workon %(projet)s' % env):
            sp_path = run('cdsitepackages ; pwd')
            import coop
            coop_path = coop.__path__[0]
            wsgi_context = {
                'site-packages': sp_path,
                'user': env.user,
                'projet': env.projet
            }
        upload_template('%s/fab_templates/wsgi.txt' % coop_path,
                        '/home/%(user)s/projects/%(projet)s/coop_local/wsgi.py' % env,
                        context=wsgi_context, use_sudo=True)
        print(green('Script WSGI pour %(projet)s créé.' % env))
    else:
        print(yellow('Script WSGI pour %(projet)s déjà existant.' % env))


def django_project():
    '''Créer un projet django dans son virtualenv'''
    with settings(show('user'), hide('warnings', 'running', 'stdout', 'stderr')):
        if not exists('/home/%(user)s/.virtualenvs/%(projet)s' % env):
            # if confirm('Pas de virtualenv "%(projet)s", faut-il le créer ?' % env, default=False):
            run('mkvirtualenv %(projet)s' % env)
            run('source .bash_profile')
        with prefix('workon %(projet)s' % env):
            run('pip install django')
            pretty_pip('psycopg2', 'Pillow')
            print(green('Django installé.'))
        if not exists('projects/%s/' % (env.projet)):
            print(yellow('le projet %(projet)s n’existe pas encore' % env))
            if confirm('Créer un projet django nommé "%(projet)s" ?' % env,
                        default=False):
                with cd('projects/'):
                    with prefix('workon %(projet)s' % env):
                        run('django-admin.py startproject %s' % (env.projet))
                        with cd('%(projet)s' % env):
                            run('mkdir logs')
                        print(green('Projet Django "%(projet)s" : OK.' % env))

        else:
            print(green('Projet Django nommé "%(projet)s" : OK.' % env))


def first_syncdb():
    project()
    with cd('projects/%(projet)s' % env):
        with prefix('workon %(projet)s' % env):
            run('./manage.py syncdb --all --noinput')
            run('./manage.py migrate --fake')
            # for f in ('auth_users.json','coop_geo.json','coop_local.json'):
            #     run('./manage.py loaddata fixtures/'+f)


def apache_nginx():
    '''Apache + mod_wsgi pour Django avec Nginx en proxy'''
    icanhaz.deb.packages(['apache2', 'libapache2-mod-wsgi'])
    with cd('/etc/apache2/'):
        if not contains('ports.conf', '127.0.0.1', use_sudo=True):
            sed('ports.conf', 'NameVirtualHost \\*:80', 'NameVirtualHost 127.0.0.1:80', use_sudo=True)
            sed('ports.conf', 'Listen 80', 'Listen 127.0.0.1:80', use_sudo=True)
            print(green('/etc/apache2/ports.conf updated'))
    with cd('/etc/apache2/'):
        if not contains('apache2.conf', 'ServerName localhost', use_sudo=True):
            sudo("echo 'ServerName localhost'|cat - apache2.conf > /tmp/out && mv /tmp/out apache2.conf")
            sudo("sed -i 's/KeepAlive On/KeepAlive Off/g' apache2.conf")
            print(green('/etc/apache2/apache2.conf updated'))
    #sudo("apache2ctl graceful") #plante de toute façon sans virtualhosts
    pretty_apt(['nginx'])
    with cd('/etc/nginx/'):
        if not contains('nginx.conf', 'worker_processes 2;', use_sudo=True):
            sudo("sed -i 's/worker_processes 4;/worker_processes 2;/g' nginx.conf")
            print(green('/etc/nginx/nginx.conf updated'))
    #to be continued...proxy.conf, vhosts...


def apache_setup():
    '''Config générale Apache + mod_wsgi sans media proxy'''
    print(yellow('Configuration d’Apache...'))
    pretty_apt(['apache2', 'libapache2-mod-wsgi'])
    #virer le site par défaut
    with cd('/etc/apache2/'):
        if not contains('apache2.conf', 'ServerName localhost', use_sudo=True):
            if not contains('apache2.conf', 'ServerName %(domaine)s' % env, use_sudo=True):
                sudo("echo 'ServerName %(domaine)s'|cat - apache2.conf > /tmp/out && mv /tmp/out apache2.conf" % env)
    with cd('/etc/apache2/sites-enabled/'):
        if exists('000-default'):
            sudo('rm 000-default')
            print(green('Site par défaut d’Apache supprimé'))


def create_pg_db():
    '''Créer une base de données postgres au nom du projet'''
    with settings(show('user'), hide('warnings', 'running', 'stdout', 'stderr')):
        project()
        icanhaz.postgres.database(env.projet, env.user, template='template_postgis', locale=env.locale)
        print(green('Création base de données PostgreSQL nommée "%(projet)s" : OK.' % env))


def virtualenv_setup():
    '''setup virtualenv'''
    print(yellow('Environnement virtuel et dossier "projects"...'))
    icanhaz.python.package('virtualenv', use_sudo=True)
    icanhaz.python.package('virtualenvwrapper', use_sudo=True)
    #icanhaz.python.package('virtualenvwrapper.django',use_sudo=True)
    print(green('Virtualenv installé.'))
    if not 'www-data' in run('echo | groups %(user)s' % env):
        sudo('usermod -a -G www-data %(user)s' % env)
        print(green('Utilisateur %(user)s ajouté au groupe "www-data".' % env))
    if not exists('projects/'):
        run('mkdir projects .python-eggs .virtualenvs')
        sudo('chown %(user)s:www-data .python-eggs' % env)
        sudo('chgrp -R www-data projects/')
        sudo('chmod -R 2750 projects/')
        print(green('Dossier "projects" créé.'))
    # sur .bashrc et pas .bashrc
    # + fix pour https://bitbucket.org/dhellmann/virtualenvwrapper/issue/62/hooklog-permissions
    if not contains('.bashrc', 'WORKON_HOME'):
        append('.bashrc', 'if [ $USER == %(user)s ]; then' % env)
        append('.bashrc', '    export WORKON_HOME=$HOME/.virtualenvs')
        append('.bashrc', '    export PROJECT_HOME=$HOME/projects')
        append('.bashrc', '    export VIRTUALENVWRAPPER_PYTHON=/usr/bin/python')
        append('.bashrc', '    export VIRTUALENVWRAPPER_VIRTUALENV=/usr/local/bin/virtualenv')
        append('.bashrc', '    source /usr/local/bin/virtualenvwrapper.sh')
        append('.bashrc', 'fi')
        #append('.bash_profile','if [ -f ~/.bashrc ]; then') #fabric source .bash_profile, pas .bashrc
        #append('.bash_profile','    source ~/.bashrc')
        #append('.bash_profile','fi')
        append('.bash_profile', 'if [ $USER == %(user)s ]; then' % env)
        append('.bash_profile', '    export WORKON_HOME=$HOME/.virtualenvs')
        append('.bash_profile', '    export PROJECT_HOME=$HOME/projects')
        append('.bash_profile', '    export VIRTUALENVWRAPPER_PYTHON=/usr/bin/python')
        append('.bash_profile', '    export VIRTUALENVWRAPPER_VIRTUALENV=/usr/local/bin/virtualenv')
        append('.bash_profile', '    source /usr/local/bin/virtualenvwrapper.sh')
        append('.bash_profile', 'fi')
        run('source .bashrc')
        print(green('Virtualenv et Virtualenvwrapper configurés.'))
    # stop warning from bitbucket https://bitbucket.org/site/master/issue/2780/getting-warning-while-using-https-and-ssh
    if not contains('.hgrc', 'bitbucket.org'):
        append('.hgrc', '[hostfingerprints]')
        append('.hgrc', 'bitbucket.org = 24:9c:45:8b:9c:aa:ba:55:4e:01:6d:58:ff:e4:28:7d:2a:14:ae:3b')


def push():
    '''Pull code from github on the server'''
    gitsrv = 'git+git://github.com/quinode/'
    app = prompt('app to update ?', default='django-coop')
    #with settings(show('user'),hide('warnings', 'running', 'stdout', 'stderr')):
    with prefix('workon coop'):
        try:
            print(yellow('desinstallation du paquet...'))
            run('pip uninstall -q %s' % app)
        except:
            print(red('paquet non installé'))
            pass
        print(yellow('réinstallation du paquet depuis github...'))
        run('pip install %s%s.git' % (gitsrv, app))
        if app == 'django-coop':
            for site in ('apeas', 'alliance', 'credis', 'extramarche', 'mesclun', 'varennes'):
                with cd('projects/%s' % site):
                    with settings(warn_only=True):
                        print(yellow('%s : migration de coop_local' % site))
                        sudo('python manage.py schemamigration coop_local --auto', user='admin')
                        sudo('python manage.py migrate coop_local', user='admin')
                    print(yellow('%s : collecte des fichiers statiques...' % site))
                    sudo('python manage.py collectstatic --noinput', user='admin')
                    sudo('python manage.py clean_pyc', user='admin')

    sudo('apachectl restart')
    print(green('Les mises à jour ont été appliquées.'))


def dependencies():
    '''Vérification des modules nécessaires au projet'''
    with settings(show('user'), hide('warnings', 'running', 'stdout', 'stderr')):
        with cd('projects/%(projet)s' % env):
            if exists('requirements.txt'):
                print(yellow('Installation des dépendances du projet...'))
                with prefix('workon %(projet)s' % env):
                    with settings(show('running', 'stdout', 'stderr')):
                        run('pip install -r requirements.txt')
            else:
                print(red('Aucun fichier "requirements.txt" trouvé.'))


def locale():
    '''Règlage des locale de l'OS'''
    locale = run("echo $LANG")
    if(locale != env.locale):
        sudo('locale-gen ' + env.locale)
        sudo('/usr/sbin/update-locale LANG=' + env.locale)
        print(green('Locale mise à jour.'))


# PLUS BESOIN

# @task
# def postgis_template():
#     '''creation du template postgis - pour PostGIS 1.5 et Ubuntu > 10.10'''
#     if not 'template_postgis' in run('echo|psql -l'):
#         run('createdb -E UTF8 template_postgis')
#         run('createlang -d template_postgis plpgsql')
#         run('''psql -d postgres -c "UPDATE pg_database SET datistemplate='true' WHERE datname='template_postgis';"''')
#         run('psql -d template_postgis -f `pg_config --sharedir`/contrib/postgis-1.5/postgis.sql')
#         run('psql -d template_postgis -f `pg_config --sharedir`/contrib/postgis-1.5/spatial_ref_sys.sql')
#         run('psql -d template_postgis -c "GRANT ALL ON geometry_columns TO PUBLIC;"')
#         run('psql -d template_postgis -c "GRANT ALL ON geography_columns TO PUBLIC;"')
#         run('psql -d template_postgis -c "GRANT ALL ON spatial_ref_sys TO PUBLIC;"')
#         print(green('template_postgis pour PostgreSQL créé'))

# PLANTE ?

# @task
# def pg_access():
#     '''Access from external IP to PGSQL - setup your firewall'''
#     with cd('/etc/postgresql/9.1/main/'):
#         change = False
#         if not contains('pg_hba.conf', 'local   all         all                               password', use_sudo=True):
#             sudo("sed -i 's/local   all         all                               peer/local   all         all                               password/g' pg_hba.conf", user='postgres')
#             change = True
#             print(green('Accés local PostgreSQL pour Django'))
#         if not contains('pg_hba.conf', '0.0.0.0'):
#             # FIX : teste rate toujours , pourquoi ? - mais pas de double écriture grace à append()
#             append('pg_hba.conf', 'host    all         all         0.0.0.0/0             md5', use_sudo=True)
#             change = True
#             print(green('Accés à PostgreSQL via interfaces IP externes'))
#         if not contains('postgresql.conf', 'listen_addresses = \'\*\'', use_sudo=True):
#             sudo('echo "listen_addresses = \'*\'"|cat - postgresql.conf > /tmp/out && mv /tmp/out postgresql.conf')
#             change = True
#             print(green('PostgreSQL écoute sur toutes les interfaces IP'))
#         if change:
#             sudo('/etc/init.d/postgresql restart')


# Faut que le site soit sous GIT

# @task
# def update():
#     '''Met à jour un serveur depuis le depot git "origin"'''
#     project()
#     #with settings(show('user'),hide('warnings', 'running', 'stdout', 'stderr')):
#     with prefix('workon %(projet)s' % env):
#         dependencies()
#         with cd('projects/%(projet)s' % env):
#             print(yellow('Synchronisation du dépôt git...'))
#             run('git pull origin master')
#             print(yellow('Django : synchronisation des nouveaux modèles...'))
#             run('python manage.py syncdb --noinput')
#             print(yellow('South : applications des migrations...'))
#             run('python manage.py migrate')
#             print(yellow('Django : Collecte des fichiers statiques...'))
#             run('python manage.py collectstatic --noinput')
#     sudo('apachectl restart')
#     print(green('Les mises à jour ont été appliquées.'))



# -*- coding:utf-8 -*-
# Django settings for {{ project_name }} project.

from django.core.exceptions import ImproperlyConfigured

DEBUG = False
TEMPLATE_DEBUG = DEBUG

SECRET_KEY = '{{ secret_key }}'

import os.path
PROJECT_PATH = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
#PROJECT_NAME = PROJECT_PATH.split('/')[-1]
PROJECT_NAME = '{{ project_name }}'  # which one is safer ?


# import all default app settings from django-coop app

try:
    from coop.default_project_settings import *
except ImportError, exp:
    raise ImproperlyConfigured("Unable to find default_project_settings.py file from django-coop")

# override settings when needed in local_settings.py

try:
    from local_settings import *
except ImportError, exp:
    pass


# db_settings file for DATABASES, and CACHE backend settings

try:
    from db_settings import *
except ImportError, exp:
    raise ImproperlyConfigured("No db_settings.py file was found")


# common db_settings file for DATABASE ROUTERS

try:
    from coop.db_settings import *
except ImportError, exp:
    raise ImproperlyConfigured("Unable to find db_settings.py file from django-coop")





# do not remove : !-- creator:coop-admin --!
# WSGI config for {{ project_name }} project.

ALLDIRS = ['{{ project_alldirs }}']

# the above directory depends on the location of your python installation.
# if using virtualenv, it will need to match your projects locale.
import os
import sys
import site

prev_sys_path = list(sys.path)

for directory in ALLDIRS:
    site.addsitedir(directory)
    new_sys_path = []
    for item in list(sys.path):
        if item not in prev_sys_path:
            new_sys_path.append(item)
            sys.path.remove(item)
            sys.path[:0] = new_sys_path

sys.path.append('{{ project_directory }}')
os.environ.setdefault("DJANGO_SETTINGS_MODULE", "coop_local.settings")

# This application object is used by any WSGI server configured to use this
# file. This includes Django's development server, if the WSGI_APPLICATION
# setting points here.

from django.core.wsgi import get_wsgi_application

try:
    from raven.contrib.django.middleware.wsgi import Sentry
    application = Sentry(get_wsgi_application())
except ImportError:
    application = get_wsgi_application()

# Apply WSGI middleware here.
# from helloworld.wsgi import HelloWorldApplication
# application = HelloWorldApplication(application)



# -*- coding:utf-8 -*-

from django.conf import settings
from coop.models import StaticURIModel, URIModel
from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.contenttypes import generic


# ----------- Customizing coop-cms Article for more coop intergration

if "coop_cms" in settings.INSTALLED_APPS:

    from coop_cms.models import BaseArticle, BaseNavTree

    class CoopArticle(BaseArticle, StaticURIModel):

        organization = models.ForeignKey('coop_local.Organization', blank=True, null=True,
                                verbose_name=_('publisher'), related_name='articles')
        person = models.ForeignKey('coop_local.Person', blank=True, null=True,
                                    verbose_name=_(u'person'), related_name='articles')

        remote_person_uri = models.CharField(_(u'person URI'), blank=True, max_length=200, editable=False)
        remote_person_label = models.CharField(_('person'), blank=True, max_length=250)

        remote_organization_uri = models.CharField(_(u'organization URI'), blank=True, max_length=200, editable=False)
        remote_organization_label = models.CharField(_('organization'), blank=True, max_length=250)

        if "coop.agenda" in settings.INSTALLED_APPS:
            dated = generic.GenericRelation('coop_local.Dated')

        def label(self):
            return self.title

        # def can_publish_article(self, user):
        #     return (self.author == user)

        #def can_edit_article(self, user):
        #   return True
        #   test on URI, not on django user

        class Meta:
            verbose_name = _(u"article")
            verbose_name_plural = _(u"articles")
            abstract = True
            app_label = 'coop_local'


# ----------- Idem for coop-cms NavTree            

    class CoopNavTree(BaseNavTree, URIModel):

        class Meta:
            verbose_name = _(u'Navigation tree')
            verbose_name_plural = _(u'Navigation trees')
            abstract = True
            app_label = 'coop_local'





# -*- coding:utf-8 -*-

from django.shortcuts import render_to_response, redirect
from coop_local.models import Organization
from django.template import RequestContext

# def org_list(request):
#     context = {}
#     context['org_list'] = Organization.objects.filter(active=True).order_by('statut')
#     return render_to_response('org/org_list.html',context,RequestContext(request))



class GEOFLARouter(object):

    def db_for_read(self, model, **hints):
        if model._meta.app_label == 'geodjangofla':
            return 'geofla_db'
        else:
            return 'default'

    def db_for_write(self, model, **hints):
        if model._meta.app_label == 'geodjangofla':
            return None
        else:
            return 'default'

    def allow_relation(self, obj1, obj2, **hints):
        if obj1._meta.app_label == 'geodjangofla' or obj2._meta.app_label == 'geodjangofla':
            return None
        else:
            return True

    def allow_syncdb(self, db, model):
        if db == 'geofla_db':
            return None
        if db == 'default' and model._meta.app_label == 'feincms':
            return False
        else:
            return True


DATABASE_ROUTERS = ['coop.db_settings.GEOFLARouter']



# -*- coding:utf-8 -*-
"""
As DistributionTasks.objects.process is call on post_save signal (when a DistributionTasks) is created.
It is always possible that a Tasks could be distributed ... timeout, exception, ....error 500
This job is in charge to try again.
Redis Failed queue should also be handled.

At least, dequeud

"""

import logging
from django.conf import settings
from django_extensions.management.jobs import QuarterHourlyJob
import subhub

# As subhub is the only app the use redis. All Redis stuff are logger 
# subhub.mainteance  log
log = logging.getLogger('subhub.maintenance')


class Job(QuarterHourlyJob):
    help = "subhub DistributionTasks process"

    def execute(self):
        dt = subhub.models.DistributionTask.objects.all()
        if len(dt) > 0 and settings.SUBHUB_MAINTENANCE_AUTO:
            subhub.models.DistributionTask.objects.process(log=log)






# -*- coding:utf-8 -*-
from django.utils.translation import ugettext_lazy as _
from django.core.management.base import CommandError
from django.core.management.templates import TemplateCommand
from django.utils.crypto import get_random_string
from django.utils.importlib import import_module
import os
from optparse import make_option


class Command(TemplateCommand):
    help = _(u"Creates a COOP project directory structure for the given "
            "project name in the current directory or optionally in the "
            "given directory.")

    option_list = TemplateCommand.option_list + (
        make_option('--redis_port',
                    action='store', dest='redis', default=6379,
                    help=_(u'Please set the redis port the application will used (default value is 6379')),
        make_option('--redis_bin',
                    action='store', dest='redis_bin', default='/opt/redis/bin/',
                    help=_(u'Please set the redis bin directory (default value is /opt/redis/bin/')),

        make_option('--domain',
                    action='store', dest='domain', default='localhost:8000',
                    help=_(u'Please set he domain name of your coop application (default value is localhost:8000/')),


        )

    def handle(self, project_name=None, target=None, *args, **options):
        if project_name is None:
            raise CommandError("you must provide a project name")

        # Check that the project_name cannot be imported.
        try:
            import_module(project_name)
        except ImportError:
            pass
        else:
            raise CommandError("%r conflicts with the name of an existing "
                               "Python module and cannot be used as a "
                               "project name. Please try another name." %
                               project_name)

        # Create a random SECRET_KEY hash to put it in the main settings.
        chars = 'abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)'
        options['secret_key'] = get_random_string(50, chars)

        import coop
        coop_dir = coop.__path__[0]

        # options dict is add to the context. Thus it is used to add new template
        # variables. Here 'template' is set 'hardy'
        options['template'] = os.path.join(coop_dir, 'templates', 'project_template')
        options['project_alldirs'] = os.path.split(coop_dir)[0]
        # For supervisor.conf file
        options['extensions'].extend(['conf', 'json', 'txt'])
        options['virtualenv'] = os.getenv('VIRTUAL_ENV')
        p = os.popen('which runinenv.sh', "r")
        options['runinenv'] = p.readline().replace('\n', '')
        p.close()

        super(Command, self).handle('project', project_name, target, **options)



from django.conf.urls.defaults import *

from coop.agenda import views


urlpatterns = patterns('',
    #url(r'^(?:calendar/)?$',
    url(r'^calendar/?$',
        view=views.today_view,
        name='agenda-today'),

    url(r'^calendar/(?P<year>\d{4})/$',
        view=views.year_view,
        name='agenda-yearly-view'),

    url(r'^calendar/(\d{4})/(0?[1-9]|1[012])/$',
        view=views.month_view,
        name='agenda-monthly-view'),

    url(r'^calendar/(\d{4})/(0?[1-9]|1[012])/([0-3]?\d)/$',
        view=views.day_view,
        name='agenda-daily-view'),

    url(r'^events/$',
        view=views.event_listing,
        name='agenda-events'),

    url(r'^calendar/(?P<slug>[\w-]+)/$',
        view=views.calendar_listing,
        name='agenda-calendar'),

    url(r'^events/add/$',
        view=views.add_event,
        name='agenda-add-event'),

    url(r'^events/(\d+)/$',
        view=views.event_view,
        name='agenda-event'),

    url(r'^events/mini/(\d+)/$',
        view=views.event_minimal_view,
        name='event-minimal-view'),  # CBV !!!

    url(r'^events/(\d+)/(\d+)/$',
        view=views.occurrence_view,
        name='agenda-occurrence'),
)



# -*- coding:utf-8 -*-
from django.contrib import admin
from django.conf.urls.defaults import patterns, include, url

# # https://code.djangoproject.com/ticket/10405#comment:11
# from django.db.models.loading import cache as model_cache
# if not model_cache.loaded:
#     model_cache.get_models()

admin.autodiscover()


# Add you own URLs here
urlpatterns = []

# urlpatterns += patterns('',
#    url(r'^org/$', 'coop_local.views.org_list', name="org_list"), #view coop
# )


from coop.default_project_urls import *


#    url(r'^org/$', 'coop_local.views.org_list', name="org_list"), #view coop a surcharger --> voir plutot les CBV



# -*- coding:utf-8 -*-

from django.db import models
from django.db.models.loading import get_model
from django_extensions.db import fields as exfields
from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist
from django.utils.translation import ugettext_lazy as _
from extended_choices import Choices
import shortuuid
from rdflib import Graph, plugin, Literal, URIRef, ConjunctiveGraph
from django.contrib.contenttypes import generic
from django.db import IntegrityError
from django.template import Template, Context
from urlparse import urlsplit
from django_push.subscriber.models import Subscription
import feedparser
import os
import tempfile
import coop
import time
from subhub.models import DistributionTask
from django.core.validators import URLValidator
from django.core.exceptions import ValidationError



URI_MODE = Choices(
    ('LOCAL',  1, _(u'Local')),
    ('COMMON',   2, _(u'Common')),
    ('IMPORTED', 3, _(u'Imported')),
)


# Serializer
plugin.register('n3', plugin.Serializer,
         'rdflib.plugins.serializers.n3', 'N3Serializer')
plugin.register('pretty-xml', plugin.Serializer,
         'rdflib.plugins.serializers.rdfxml', 'PrettyXMLSerializer')
plugin.register('json-ld', plugin.Serializer,
        'rdflib_jsonld.jsonld_serializer', 'JsonLDSerializer')

plugin.register('trix', plugin.Serializer,
                'rdflib.plugins.serializers.trix', 'TriXSerializer')




class TimestampedModel(models.Model):
    created = exfields.CreationDateTimeField(_(u'created'), null=True)
    modified = exfields.ModificationDateTimeField(_(u'modified'), null=True)

    class Meta:
        abstract = True


class StaticURIModel(models.Model):
    """
    To use this model as a basis for your own abstract model, you need to have
    a 'uri_id' property set and a string for the model type URL representation :

    @property
    def uri_id(self):
        return self.username # example with username

    uri_registry = 'org'

    the uri_fragment can then be overriden in a real model derived from your abstract model

    """
    class Meta:
        abstract = True

    uri_mode = models.PositiveSmallIntegerField(_(u'Mode'), choices=URI_MODE.CHOICES, default=URI_MODE.LOCAL, editable=False)
    uri = models.CharField(_(u'main URI'), blank=True, null=True,
                            max_length=250, editable=False)  # FIXME : null=True incompatible with unique=True

    # Le code suivante ne marche pas avec south et pourtant il correxpond exactement à ce que je voudrais
    # faire. Si on supprime unique=True, alors la migration se passe bien, mais toutes les
    # du model ont la meme valeur.... Et bien sur, si on met uniaue=True, south rale car
    # le champs n'est pas unique.
    # South n'a pas l'air capable de traiter ce cas.
    # D'apres la doc, il est possible de le faire en 3 fois, sans utiliser le default, avec
    # un datamigration (voir la doc Part 3: Advanced Commands and Data Migrations)
    #uuid = models.CharField(_(u'uuid'), max_length=50, default=shortuuid.uuid, unique=True)

    # La version simple c'est de passer par le save() et de supprimer le default.... c'est pas
    # tres beau, car un plus couteux en runtime... mais bon
    uuid = models.CharField(_(u'uuid'), max_length=50, null=True, editable=False, unique=True, default=shortuuid.uuid)

    links = generic.GenericRelation('coop_local.Link', related_name="%(app_label)s_%(class)s_related")

    # the default value, this attribut should be overloaded
    domain_name = settings.DEFAULT_URI_DOMAIN
    #str(Site.objects.get_current().domain)

    def label(self):
        return "Not Yet Implemented label method"

    # This method could be overwritten by subClasses
    @property
    def uri_id(self):
        if self.uri_mode == URI_MODE.IMPORTED:
            try:
                scheme, host, path, query, fragment = urlsplit(self.uri)
                sp = path.split('/')
                return sp[len(sp) - 2]
            except Exception, e:
                raise ValueError(u'Wrong uri value for %s. Reason %s' % (self.uri, e))

        else:
            return self.uuid

    def uri_registry(self):
        return self.__class__.__name__.lower()

    def init_uri(self):
        return u"http://%s/id/%s/%s/" % (self.domain_name,
                                         self.uri_registry(),
                                         unicode(self.uri_id)  # can be anything = int, uuid, unicode...
                                        )

    # self.uri est null ou vide le creer (cad appel init_uri)
    # si le domain a changé alors changer l'uri (cad dire appel init_uri)
    # si le uri_registery a changer alors changer l'uri
    # attention dans les 2 cas le uuid ne bouge pas.... il faut alors aller le chercher
    #
    # repenser aux commons et aux importés pour voir si tout marche

    def save(self, *args, **kwargs):
         # create / update URI
        if self.uri_mode != URI_MODE.IMPORTED:
            if not self.uri or self.uri == '':
                self.uri = self.init_uri()
            elif self.uri != self.init_uri():
                self.uri = self.init_uri()  # uri_id est pas pareil
        super(StaticURIModel, self).save(*args, **kwargs)

    def toRdf(self, format):
        """
           format correspond to the standard rdflib format keyword,
               'n3' for n3
               'xml' for xml
               'json-ld' for json-ld
               'trix' for trix
        """
        # D2R SPARQL endpoint is local, served by Jetty :
        uriSparql = 'http://localhost:8080/' + settings.PROJECT_NAME + '/sparql'
        graph = ConjunctiveGraph('SPARQLStore')
        graph.open(uriSparql, False)
        graph.store.baseURI = str(uriSparql)

        # The short form of the construct where see 16.2.4 CONSTRUCT WHERE http://www.w3.org/TR/sparql11-query/#constructWhere
        cquery = "construct where { <%s> ?p ?o .} "
        res = graph.query(cquery % self.uri)

        if format == 'ttl':
            format = 'n3'
        if format == 'json':
            format = 'json-ld'
        return res.graph.serialize(format=format)

    def toN3(self):
        return self.toRdf("n3")

    def toXml(self):
        return self.toRdf("xml")

    def toJson(self):
        return self.toRdf("json-ld")

    def toTrix(self):
        return self.toRdf("trig")



    ####
    # TODO
    # The following code has to be cleaned... print, log, ...
    # Still waiting for tests


    @classmethod
    def hostNewUri(cls, old_uri, graph=None):
        """
        This methood change the old_uri, which belong to an external domain_name
        to an uri hosted by this domain. The graph is the rdf graph of the resources.
        The triples are, of course, the following
          <old_uri> <pres1> <value1>
          ....
        but the values are supposed to be up-to-date, we can image that the administrator has change
        (at least checked) them.
        If no graph is provided, old_uri is parsed and used

        This method return the new uri or raise an exception.
        Link between old_uri and new_uri is done with the
        help of the Link model

        We suppose that only model from coop_local could be rehosted.
        """
        if not graph:
            graph = Graph()
            graph.parse(old_uri)

        # check if an instance with thus old_uri exists
        scheme, host, path, query, fragment = urlsplit(old_uri)
        model_name = path.split('/')[2]
        model = get_model('coop_local', model_name)
        if not model:
            raise Exception(_(u"Cannot find a model  for %s" % old_uri))
        try:
            instance = model.objects.get(uri=old_uri)
        except model.DoesNotExist:
            instance = model(uri=old_uri)   # old_uri is still usefull for updateFromRdf

        if instance:
            instance.updateFromRdf(graph)

        from coop_local.models import Link, LinkProperty

        instance.uuid = shortuuid.uuid()
        instance.uri_mode = URI_MODE.LOCAL
        instance.uri = instance.init_uri()
        replaces, created = LinkProperty.objects.get_or_create(uri=settings.NS.dct.replaces, label=u'replaces')
        link = Link(predicate=replaces, object_uri=old_uri)
        instance.links.add(link)
        instance.save()
        return instance.uri


    # La changement d'uri se fait en deux temps : prepareChangeDomainUri et updateDomainUri
    # - on ajoute le triplet olduri dct:isReplacedBy new_uri et on sauve, pour  notifier tous les autres
    # coop qui sont éventuellement abonnéeà cette instance
    # - on met àjour l'uri est le champs uri_mode
    def prepareUpdateDomainUri(self, new_uri):
        """
            When an instance of URIModel asks to rehost its uri to an external domaine. This method
            is called when the new uri has been created by the external domain.
        """
        from coop_local.models import Link, LinkProperty

        replacedBy, created = LinkProperty.objects.get_or_create(uri=settings.NS.dct.isReplacedBy, label=u'isReplacedBy')
        link = Link(predicate=replacedBy, object_uri=new_uri)
        self.links.add(link)
        self.save()

    def updateDomainUri(self, new_uri, sync=False):
        """
        Prerequis : self.prepareChangeDomainUri(new_uri) has been called.
        This means, all other coop which import self.uri are aware of the rehost process, they
        now point to the new_uri object. The instance hasjust to update its uri and uri mode.
        If sync is true, then update is done froom new_uri

        """
        # First check, that all Tasks have been perform. To be such that subcribers are aware of
        # the rehosted action.
        dts = DistributionTask.objects.all()
        if not dts == []:
            DistributionTask.objects.process()
        self.uri = new_uri
        self.uri_mode = URI_MODE.IMPORTED
        if sync:
            g = Graph()
            g.parse(new_uri)
            self.updateFromRdf(g)  # save is done here
        else:
            self.save()      # post_save signal is also doing the new Subcription



    @classmethod
    def getD2rqGraph(cls):
        context = {}
        context['mapping_template'] = 'd2r/' + cls.__name__.lower() + '.ttl'
        context['namespaces'] = settings.RDF_NAMESPACES

        tmpfile = os.path.dirname(os.path.abspath(coop.__file__)) + '/templates/d2r/model.ttl'
        f = open(tmpfile, 'r')
        t = Template(f.read())
        f.close()
        res = t.render(Context(context))
        (fd, fname) = tempfile.mkstemp()
        f = open(fname, 'w')
        f.write(res)
        f.close()
        graph = Graph()
        graph.parse('file:' + fname, format='n3')
        return graph

    @classmethod
    def updateFromFeeds(cls):
        print "Update feed for class name %s" % cls.__name__
        feed_url = settings.PES_HOST + 'feed/' + cls.__name__.lower() + '/'
        parsedFeed = feedparser.parse(feed_url)
        print "Parse feed %s" % feed_url
        for entry in parsedFeed.entries:
            print "Entry summary is %s " % entry.summary
            fd, fname = tempfile.mkstemp()
            os.write(fd, entry.summary)
            os.close(fd)
            g = Graph()
            g.parse(fname, format="json-ld")
            done = set()
            for subj in g.subjects(None, None):
                if not (subj in done):
                    instance = cls.objects.get(uri=str(subj))
                    if not instance:
                        print "Nothing to do with uri %s. Not found in database" % subj
                    else:
                        instance.updateFromRdf(g)
                    done.add(subj)

    # The "reverse mapping is done here"
    def updateFromRdf(self, graph):
        # Before reversing mapping, we have to check that the uri has not been
        # changed. This the case for URI_MODE.IMPORTED instance which still point to
        # the old uri

        replacedBy = list(graph.objects(URIRef(self.uri), settings.NS.dct.isReplacedBy))
        if self.uri_mode == URI_MODE.IMPORTED and len(replacedBy) == 1:
            print "Handle rehost imported uri"
            g = Graph()
            g.parse(replacedBy[0])
            self.uri = replacedBy[0]
            self.updateFromRdf(g)
        else:
            db_table = self.__class__._meta.db_table
            d2rqGraph = self.__class__.getD2rqGraph()
            for field in self._meta.fields:
                dbfieldname = db_table + '.' + field.name
                pred = checkDirectMap(dbfieldname, d2rqGraph)
                if pred:
                    update = list(graph.objects(URIRef(self.uri), pred))
                    if len(update) > 1:
                        print "    The field %s cannot be updated. Too many values" % dbfieldname
                    elif len(update) == 0:
                        print "Nothing to update for field %s" % dbfieldname
                    else:
                        print "For id %s update the field %s" % (self.id, dbfieldname)
                        update = update[0]
                        # Well some field have to be prepared
                        if isinstance(field, models.fields.DateField):
                            update = update.split('.')[0]
                            try:
                                tt = time.strptime(update, "%Y-%m-%dT%H:%M:%S")
                            except ValueError:
                                tt = time.strptime(update, "%Y-%m-%d")
                            update = "%d-%d-%d" % (tt.tm_year, tt.tm_mon, tt.tm_mday)
                        setattr(self, field.name, update)
                elif isinstance(field, models.ForeignKey):
                    print "try an FKEY"
                    pred = checkDirectMapFK(dbfieldname, d2rqGraph)
                    if pred:
                        self.updateFKfromRdf(field, dbfieldname, pred, graph)
                    else:
                        print "    The field %s cannot be updated." % dbfieldname

                else:
                     # See org/models file to have example of "special" fiels handling
                    if hasattr(self, 'updateField_' + field.name):
                        getattr(self, 'updateField_' + field.name)(dbfieldname, graph)
                    else:
                        print "    The field %s cannot be updated." % dbfieldname
            print "UpdateFromRdf all fields have been handled"
            self.save()
            print "save done"


    def updateFKfromRdf(self, field, dbfieldname, pred, graph):
        rdfObj = list(graph.objects(URIRef(self.uri), pred))
        if len(rdfObj) == 1:
            rdfObj = rdfObj[0]
            print "FK object found %s" % rdfObj
            if isinstance(rdfObj, URIRef):
                try:
                    obj = field.related.parent_model.objects.get(uri=str(rdfObj))
                    setattr(self, field.name + '_id', obj.id)
                    print "For id %s update the field %s" % (self.id, dbfieldname)
                except ObjectDoesNotExist:
                    # TODO: Object not found in db.... here we have to check il it
                    # has to be imported or not.
                    print "Obj not in bd : %s" % rdfObj
                    print u"    The field %s cannot be updated." % dbfieldname
            else:
                print "    The field %s cannot be updated." % dbfieldname
        else:
            print "    The field %s cannot be updated." % dbfieldname

        pass


    # This a very simple case, where feed and ub share the same host
    def subscribeToUpdades(self, host=settings.PES_HOST):
        feed_url = "http://%s/feed/%s/%s/" % (host, self.__class__.__name__.lower(), self.uri_id)
        validate = URLValidator(verify_exists=True)
        try:
            validate(feed_url)
            print u"Try to subscribe to feed %s" % feed_url
            Subscription.objects.subscribe(feed_url, hub="http://%s/hub/" % host)
        except ValidationError, e:
            print u" Imposible to subscribe to %s : %s" % (feed_url, e)


    def unsubscribeToUpdades(self, host=settings.PES_HOST):
        feed_url = "http://%s/feed/%s/%s/" % (host, self.__class__.__name__.lower(), self.uri_id)
        validate = URLValidator(verify_exists=True)
        try:
            validate(feed_url)
            print u"Try to subscribe to feed %s" % feed_url
            Subscription.objects.unsubscribe(feed_url, hub="http://%s/hub/" % host)
        except ValidationError, e:
            print u" Imposible to unsubscribe to %s : %s" % (feed_url, e)


class URIModel(StaticURIModel, TimestampedModel):
    class Meta:
        abstract = True


def checkDirectMap(dbfieldName, d2rqGraph):
    lit = Literal(dbfieldName)
    subj = list(d2rqGraph.subjects(settings.NS.d2rq.column, lit))
    if len(subj) != 1:
        subj = list(d2rqGraph.subjects(settings.NS.d2rq.uriColumn, lit))
        if len(subj) != 1:
            return None
    subj = subj[0]
    prop = list(d2rqGraph.objects(subj, settings.NS.d2rq.property))
    if len(prop) != 1:
        return None
    else:
        return prop[0]


def checkDirectMapFK(dbfieldName, d2rqGraph):
    # on commence par chercher tous les triples de la forme
    # select * where { ?s d2rq:join ?l. filter(?l contains dbfieldname) }
    # mais comme le d2rqGraph n'est pas un SparqlGraph on le fait à la main

    # Well it is not possible to use settings.NS.d2rq.join because
    # there is a conflict of names with the join function of the Namespace
    # class....
    tr = list(d2rqGraph.triples((None, URIRef(str(settings.NS.d2rq) + 'join'), None)))

    def useField((s, p, o)):
        if isinstance(o, Literal):
            return dbfieldName in o
        else:
            return False
    candidate = filter(useField, tr)
    if len(candidate) == 1:
        print "Found candidate"
        subj = candidate[0][0]
        prop = list(d2rqGraph.objects(subj, settings.NS.d2rq.property))
        if len(prop) == 1:
            return prop[0]
        else:
            return None
    else:
        return None





# It seems to be the best place to do the connection
# see http://stackoverflow.com/questions/7115097/the-right-place-to-keep-my-signals-py-files-in-django
from django.core.signals import  request_finished
from coop.signals import post_save_callback, post_delete_callback, listener
from django_push.subscriber.signals import updated

request_finished.connect(post_save_callback, sender=StaticURIModel)
request_finished.connect(post_delete_callback, sender=StaticURIModel)
updated.connect(listener)







# -*- coding:utf-8 -*-

SITE_AUTHOR = 'Organisme'
SITE_TITLE = 'Demo Django-coop'
DEFAULT_URI_DOMAIN = '{{ domain }}'

DEBUG = False
TEMPLATE_DEBUG = DEBUG

ADMINS = (
    ('Administrateur', 'web@quinode.fr'),
)

MANAGERS = ADMINS
SEND_BROKEN_LINK_EMAILS = True
INTERNAL_IPS = ('127.0.0.1', '92.243.30.98')


SUBHUB_MAINTENANCE_AUTO = False    # set this value to True to automatically syncronize with agregator
PES_HOST = 'pes.domain.com'         
THESS_HOST = 'thess.domain.com'

from django.conf import settings

INSTALLED_APPS = settings.INSTALLED_APPS + [
    # select your coop components
    'coop.tag',
    'coop.agenda',
    'coop.article',
    'coop.mailing',
    'coop.exchange',
    #'coop.webid',
    'coop_geo',
    'coop_local',
]